# Web3 utils
from RupineWeb3Utils import web3_checker
from RupineWeb3Utils import web3_generic
from RupineWeb3Utils import web3_globals
from RupineWeb3Utils import web3_txScan
from RupineWeb3Utils import web3_provider

# Utils
#from RupineWeb3Utils.utils import helper
#from RupineWeb3Utils.utils import payloadBuilder
#from RupineWeb3Utils.utils import ObjToken

# Slack
#from RupineWeb3Utils.slack import SlackPost

# Rupine API
#from RupineWeb3Utils.rupine_api import credentials
#from RupineWeb3Utils.rupine_api import tokenMetaData



from itertools import chain
import os
from web3 import Web3
from RupineWeb3Utils.rupine_db import herokuCredentials, herokuDbAccess
from RupineWeb3Utils.web3_utils import web3_globals

def GetProviderAnkr(chain_id, connection, schema):
    username = herokuCredentials.getCredential(connection, schema, "ankr_prod", "USERNAME", 1) 
    password = herokuCredentials.getCredential(connection, schema, "ankr_prod", "PASSWORD", 1)
    url = herokuCredentials.getCredential(connection, schema, "ankr_prod", "URL", chain_id)

    endpointQuery = url.format(':'.join([username,password]))
    return Web3(Web3.HTTPProvider(endpointQuery))

def GetProviderGetBlock(chain_id, connection, schema):
    if chain_id == 1:
        api_key_getblock = herokuCredentials.getCredential(connection, schema, "GetBlock", "API KEY", 1) 
        url = 'https://eth.getblock.io/mainnet/?api_key={0}'
        endpoint = url.format(api_key_getblock)
        return Web3(Web3.HTTPProvider(endpoint))
    return None

def GetAllProvider(chain_id,prodEnv:bool=True):
    w3s = []
    #if chain_id == 1:
    #    w3s.append(GetProviderGetBlock(chain_id))

    w3s.append(GetProviderAnkr(chain_id,prodEnv))
    return w3s


import json
from web3 import Web3
from web3.contract import Contract

def get_contract_event_in_transaction(contract:Contract, txHash:str,eventName:str):

    # Web3.py scheint es nicht zu können events string basiert abzurufen.
    # e.g. contract.events['Transfer']
    # Es ist notwendig immer einer Funktionsaufruf abzusetzen
    # e.g. contract.events.Transfer()
    # Um dies zu umschiffen wird hier dynamisch Code erzeugt und ausgeführt

    code = '''def getEvent():
    try:
        c = contract.events.{0}()
        return c
    except:
        return None
c = getEvent()
        '''.format(eventName)

    d = {}
    exec(code, {'contract': contract}, d)
    contractEvent = d['c']
    return contractEvent

# returns <tokenClass>,<verified>
def check_contract_standard(contract:Contract):
    if not contract:
        return ['n/a', 'N']
    if not contract.abi:
        return ['n/a', 'N']

    # Check ABI if ERC20 or ERC721 interface  functions are implemented with correct name, input and output parameter
    # ERC20 check according to: https://eips.ethereum.org/EIPS/eip-20
    abi = contract.abi

    #print(abi)

    ERC20Checklist = []
    # Mandafory: Function totalSupply
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'totalSupply'"})
    # Mandafory: Function BalanceOf
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'balanceOf'"})
    # Mandafory: Function transfer
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'transfer'"})
    # Mandafory: Function allowance
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'allowance'"})
    # Mandafory: Function approve
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'approve'"})
    # Mandafory: Function transferFrom
    ERC20Checklist.append({ "Check": False, "type": "'type': 'function'", "name": "'name': 'transferFrom'"})
    # Mandafory: Event Transfer
    ERC20Checklist.append({ "Check": False, "type": "'type': 'event'", "name": "{'anonymous': False, 'inputs': [{'indexed': True, 'internalType': 'address', 'name': 'from', 'type': 'address'}, {'indexed': True, 'internalType': 'address', 'name': 'to', 'type': 'address'}, {'indexed': False, 'internalType': 'uint256', 'name': 'value', 'type': 'uint256'}], 'name': 'Transfer', 'type': 'event'}"})
    # Mandafory: Event Approval
    ERC20Checklist.append({ "Check": False, "type": "'type': 'event'", "name": "{'anonymous': False, 'inputs': [{'indexed': True, 'internalType': 'address', 'name': 'owner', 'type': 'address'}, {'indexed': True, 'internalType': 'address', 'name': 'spender', 'type': 'address'}, {'indexed': False, 'internalType': 'uint256', 'name': 'value', 'type': 'uint256'}], 'name': 'Approval', 'type': 'event'}"})

    ERC721Checklist = []
    ERC721Checklist.append({ "Check": False, "name": "some placeholder here"})

    for line in abi:
        for item in ERC20Checklist:
            if  str(item['name']) in str(line) and str(item['type']) in str(line):
                item['Check'] = True
        for item in ERC721Checklist:
            #print(line)
            #print("..................")
            #print(item['ABILine'])
            #print("-----------------------------")
            #print("NEXT LINE")
            #print("-----------------------------")
            if  str(item['name']) in str(line) and str(item['type']) in str(line):
                item['Check'] = True
            
    isERC20 = True
    for item in ERC20Checklist:
        isERC20 = isERC20 and item['Check']
    if isERC20:
        return ['ERC20','Y']

    isERC721 = True
    for item in ERC20Checklist:
        isERC721 = isERC721 and item['Check']
        #if item['Check']:
        #    print('Check')
        #else:
        #    print(item['ABILine'])
    if isERC721:
        return ['ERC20','Y']

    return ['n/a','Y']
